#!/usr/bin/python3

import json
import os
import psycopg2
import sys
from psycopg2.extras import RealDictCursor
from importlib.machinery import SourceFileLoader
import reconfigurefews

def translateColToPg(column_name): # model/Parameter.js to postgres; currently no translation required
    return column_name

def translateValToPg(column_name, value):
    if (not value is None) and (str(value).strip() != ''):
        if column_name == 'groupkey' or column_name == 'valueresolution' or column_name == 'featureclasskey':
            return str(value).strip()
        else:
            return "'" + str(value).strip() + "'"
    else:
        return "NULL"

def updateParameters(parameterInserts, parameterUpdates):
    success = True
    messages = []
    file = open("../.credentials/.postgresadmin")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    try:
        sql = "BEGIN;"
        query.execute(sql)
        parameters = {}
        for parameterkey in parameterInserts.keys():
            sql = ""
            inserts = parameterInserts[parameterkey]
            for column_name in inserts.keys():
                if sql == "":
                    sql += "INSERT INTO fews.parameterstable ("
                else:
                    sql += ", "
                sql += translateColToPg(column_name)
            sql += ") VALUES ("
            first = True
            for column_name in inserts.keys():
                if first:
                    first = False
                else:
                    sql += ", "
                sql += translateValToPg(column_name, inserts[column_name])
            sql += ") RETURNING parameterkey;"
            query.execute(sql)
            parameterkey = query.fetchone()['parameterkey']
            parameter = {}
            parameter['parameterkey'] = parameterkey
            parameters[parameterkey] = parameter
        for parameterkey in parameterUpdates.keys():
            sql = ""
            updates = parameterUpdates[parameterkey]
            for column_name in updates.keys():
                if sql == "":
                    sql += "UPDATE fews.parameterstable SET "
                else:
                    sql += ", "
                sql += translateColToPg(column_name) + "=" + translateValToPg(column_name, updates[column_name])
            sql += " WHERE parameterkey=" + str(parameterkey) + ";"
            query.execute(sql)
        messages = list(parameters.values())
        reconfigurefews.probeaccess() # an exception here will rollback the transaction
        sql = "COMMIT;"
        query.execute(sql)
        
    except Exception as err:
        success = False
        messages.append(str(err) + ".")
        messages.append("Failed to update the majisys parameters!")
    return success, messages

contentlength = os.environ.get('CONTENT_LENGTH', 0)
postdata = sys.stdin.read(int(contentlength))
postdata = json.loads(postdata)
ticket = postdata['ticket']
curpath = os.path.dirname(os.path.realpath( __file__ ))
cas = SourceFileLoader("cas", curpath + "/../management/cas.py").load_module()
if cas.authorized(ticket,{"basedata" : "rw"}):
    parameterUpdates = {}
    parameterInserts = {}
    for record in postdata:
        if record == 'ticket':
            continue
        record = postdata[record]
        if 'parameterkey' in record: # insert or update
            parameterkey = record['parameterkey']
            if parameterkey < 0: # insert
                inserts = {}
                for key in record.keys():
                    if key != 'parameterkey':
                        inserts[key] = record[key]
                        if key == 'id':
                            inserts['shortname'] = record[key]
                parameterInserts[parameterkey] = inserts
            else: # update
                updates = {}
                for key in record.keys():
                    if key != 'parameterkey':
                        updates[key] = record[key]
                        if key == 'id':
                            updates['shortname'] = record[key]
                parameterUpdates[parameterkey] = updates

    success, messages = updateParameters(parameterInserts, parameterUpdates)
else:
    success = False
    messages = ['Not authorized.']

print ("Content-type: application/json")
print ()
if success:
    reconfigurefews.updatefewsxml()
    print("{\"success\": true, \"messages\": " + json.dumps(messages) + "}")
else:
    print("{\"success\": false, \"messages\": " + json.dumps(messages) + "}")

