#!/usr/bin/python3

import json
import os
import psycopg2
import sys
from psycopg2.extras import RealDictCursor
from importlib.machinery import SourceFileLoader

def translateColToPg(column_name): # model/ParameterGroup.js to postgres; currently no translation required
    return column_name

def translateValToPg(column_name, value):
    if (not value is None) and (str(value).strip() != ''):
        return "'" + str(value).strip() + "'"
    else:
        return "NULL"

def updateGroups(groupInserts, groupUpdates):
    success = True
    messages = []
    file = open("../.credentials/.postgresadmin")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    try:
        sql = "BEGIN;"
        query.execute(sql)
        groups = {}
        for groupkey in groupInserts.keys():
            sql = ""
            inserts = groupInserts[groupkey]
            for column_name in inserts.keys():
                if sql == "":
                    sql += "INSERT INTO fews.parametergroups ("
                else:
                    sql += ", "
                sql += translateColToPg(column_name)
            sql += ") VALUES ("
            first = True
            for column_name in inserts.keys():
                if first:
                    first = False
                else:
                    sql += ", "
                sql += translateValToPg(column_name, inserts[column_name])
            sql += ") RETURNING groupkey;"
            query.execute(sql)
            groupkey = query.fetchone()['groupkey']
            group = {}
            group['groupkey'] = groupkey
            groups[groupkey] = group
        for groupkey in groupUpdates.keys():
            sql = ""
            updates = groupUpdates[groupkey]
            for column_name in updates.keys():
                if sql == "":
                    sql += "UPDATE fews.parametergroups SET "
                else:
                    sql += ", "
                sql += translateColToPg(column_name) + "=" + translateValToPg(column_name, updates[column_name])
            sql += " WHERE groupkey=" + str(groupkey) + ";"
            query.execute(sql)
        messages = list(groups.values())
        sql = "COMMIT;"
        query.execute(sql)
        
    except Exception as err:
        success = False
        messages.append(str(err) + ".")
        messages.append("Failed to update the majisys unit definitions!")
    return success, messages

contentlength = os.environ.get('CONTENT_LENGTH', 0)
postdata = sys.stdin.read(int(contentlength))
postdata = json.loads(postdata)
ticket = postdata['ticket']
curpath = os.path.dirname(os.path.realpath( __file__ ))
cas = SourceFileLoader("cas", curpath + "/../management/cas.py").load_module()
if cas.authorized(ticket,{"basedata" : "rw"}):
    groupUpdates = {}
    groupInserts = {}
    for record in postdata:
        if record == 'ticket':
            continue
        record = postdata[record]
        if 'groupkey' in record: # insert or update
            groupkey = record['groupkey']
            if groupkey < 0: # insert
                inserts = {}
                for key in record.keys():
                    if key != 'groupkey':
                        inserts[key] = record[key]
                        if key == 'id':
                            inserts['name'] = record[key]
                        elif key == 'unit':
                            inserts['displayunit'] = record[key]
                        elif key == 'displayunit':
                            inserts['unit'] = record[key]
                if not 'parametertype' in inserts:
                    inserts['parametertype'] = 0 # 0="instantaneous", 1="accumulative", 2="mean" (see https://publicwiki.deltares.nl/display/FEWSDOC/03+Parameters)
                groupInserts[groupkey] = inserts
            else: # update
                updates = {}
                for key in record.keys():
                    if key != 'groupkey':
                        updates[key] = record[key]
                        if key == 'id':
                            updates['name'] = record[key]
                        elif key == 'unit':
                            updates['displayunit'] = record[key]
                        elif key == 'displayunit':
                            updates['unit'] = record[key]
                groupUpdates[groupkey] = updates

    success, messages = updateGroups(groupInserts, groupUpdates)
else:
    success = False
    messages = ['Not authorized.']

print ("Content-type: application/json")
print ()
if success:
    print("{\"success\": true, \"messages\": " + json.dumps(messages) + "}")
else:
    print("{\"success\": false, \"messages\": " + json.dumps(messages) + "}")

