#!/usr/bin/python3

import os
import cgi
import psycopg2
from datetime import datetime
import sys
import json
from psycopg2.extras import RealDictCursor
from importlib.machinery import SourceFileLoader

def cgierror(message):
    print("Content-type: text/html")
    print()
    print(message)

def getvalues(serieskeys,username):
    print("Content-type: application/json")
    print()

    parametername = ""
    unit = ""

    file = open("../.credentials/.postgres")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)
    query = pg.cursor(cursor_factory=RealDictCursor)
    
    next = False
    for key in serieskeys:
        if key == "":
            print("Empty key provided")
            return
        sql = "WITH spatialfilter AS \
                   (SELECT ST_Union(geom) AS aoi FROM app.user JOIN app.user_spatialfilter USING(userid) JOIN app.spatialfilter USING (gid) \
                    WHERE username='" + username + "') \
                SELECT locations.id AS locationid, parameterstable.name AS parametername, parametergroups.displayunit AS unit \
                FROM fews.timeserieskeys \
                JOIN fews.locations ON timeserieskeys.locationkey = locations.locationkey \
                JOIN fews.parameterstable ON timeserieskeys.parameterkey = parameterstable.parameterkey \
                JOIN fews.parametergroups ON parameterstable.groupkey = parametergroups.groupkey \
                JOIN fews.stations ON locations.id = stations.station_id \
                JOIN spatialfilter ON ST_Intersects(geom, spatialfilter.aoi) \
                WHERE timeserieskeys.serieskey = " + key
        query.execute(sql)
        if query.rowcount == 0:
            continue
        result = query.fetchall()
        locationid = result[0]['locationid']
        # if parametername == "" and unit == "":
        parametername = result[0]['parametername']
        unit = result[0]['unit']
            # print (parametername + "," + unit + ",[", end='')
        if next:    
            print(",", end='')
        else:
            print("[", end='')
            next = True
        print("{\"timeseriesid\": " + key + ",\"locationid\": \"" + locationid + "\",", end='')
        print("\"parametername\": \"" + parametername + "\",\"unit\": \"" + unit + "\"", end='')
        #sql = "SELECT datetime::text AS date, scalarvalue AS value FROM fews.timeseriesvaluesandflags WHERE serieskey = " + key + " ORDER BY 1"

        if mindate is None:
            range = """
                SELECT min(datetime) AS start, max(datetime) AS end 
                FROM fews.timeseriesvaluesandflags 
                WHERE serieskey = %s
            """ % (key)
        else:
            range = """
                SELECT '%s 00:00:00'::timestamp AS start, 
                       '%s 24:00:00'::timestamp AS end 
            """ % (mindate,maxdate)
        
        sql = """
            WITH period AS ( 
                %s
            )
            SELECT COUNT(*) 
            FROM fews.timeseriesvaluesandflags, period
            WHERE serieskey = %s 
              AND datetime <@ tsrange(period.start, period.end)
        """ % (range,key)

        query.execute(sql)
        records = query.fetchone()['count']
        
        if records <= 500:
            sql = """
                WITH period AS ( 
                    %s
                )
                SELECT to_char(datetime ,'YYYY-MM-DD HH24:MI:SS') AS date, scalarvalue AS value 
                FROM fews.timeseriesvaluesandflags, period
                WHERE serieskey = %s
                  AND datetime <@ tsrange(period.start, period.end)
                ORDER BY 1            
            """ % (range,key)
        else:
            sql = """
                WITH period AS ( 
                    %s
                ), series AS ( 
                    SELECT generate_series(period.start, period.end, (period.end - period.start)/500) AS sample 
                    FROM period 
                ) 
                SELECT DISTINCT to_char(datetime,'YYYY-MM-DD HH24:MI:SS') AS date, scalarvalue AS value 
                FROM fews.timeseriesvaluesandflags, series 
                WHERE serieskey = %s AND datetime = ( 
                    SELECT min(datetime) 
                    FROM fews.timeseriesvaluesandflags 
                    WHERE serieskey = %s AND datetime >= series.sample 
                ) ORDER BY date;
            """ % (range,key,key)

        query.execute(sql)
        result = json.dumps(query.fetchall())
        print(", \"series\":" + result.replace("'",'"') + "}", end='')
    print("]")


# main()

params = cgi.FieldStorage()
serieskeys = params.getvalue('serieskeys')
ticket = params.getvalue('ticket')
mindate = params.getvalue('mindate')
maxdate = params.getvalue('maxdate')

curpath = os.path.dirname(os.path.realpath( __file__ ))
cas = SourceFileLoader("cas", curpath + "/../management/cas.py").load_module()

if not serieskeys is None:
    serieskeys = serieskeys.split(',')

if not serieskeys is None:
    if len(serieskeys) > 0:
        if not cas.authorized(ticket,{"timeseries" : "r"}):
            cgierror("Not authorized.")
        else:
            username = cas.getusername(ticket)
            getvalues(serieskeys,username)
    else:
        cgierror("Please select at least one time series to download")
else:
    cgierror("Please select at least one time series to download")


