#!/usr/bin/python3

import os
import json
import psycopg2
from psycopg2.extras import RealDictCursor
from importlib.machinery import SourceFileLoader
import cgi

print ("Content-type: application/json")
print ()

#-- Query Parameters
params = cgi.FieldStorage()
filterBy = params.getvalue('filterby')
filterValue = params.getvalue('filtervalue')
aggFunction = params.getvalue('function')
parameterid = params.getvalue('parameterid')
ticket = params.getvalue('ticket')
curpath = os.path.dirname(os.path.realpath( __file__ ))
cas = SourceFileLoader("cas", curpath + "/../management/cas.py").load_module()
if cas.authorized(ticket,{"crosstabs" : "r"}):
    username = cas.getusername(ticket)

    # filterBy = "year"
    # filterValue = 2010
    # aggFunction = 'avg'
    # parameterid = 1

    file = open("../.credentials/.postgres")
    connection_string = file.readline()
    pg = psycopg2.connect(connection_string)

    if filterBy == "year" :

        viewQuery = "CREATE OR REPLACE VIEW monitoring.watertab AS \
            WITH \
              params AS ( \
                SELECT '%s-01-01 00:00:00'::TIMESTAMP WITHOUT TIME ZONE AS period_start, \
                       '%s-12-31 24:00:00'::TIMESTAMP WITHOUT TIME ZONE AS period_end \
              ), \
              pt AS ( \
                SELECT parameterkey \
                FROM fews.parameterstable \
                WHERE id = '%s' \
              ), \
              spatialfilter AS ( \
                SELECT ST_Union(geom) AS aoi \
                FROM app.user JOIN app.user_spatialfilter USING(userid) JOIN app.spatialfilter USING (gid) \
                WHERE username='%s' \
              ), \
              aggregates AS ( \
                SELECT t.locationkey, \
                       to_char(s.datetime, 'YYYY'::text) AS obs_year, \
                       to_char(s.datetime, 'MM'::text) AS obs_month, \
                       btrim(to_char(s.scalarvalue, '999.9999'::text))::numeric AS obs_value \
                FROM pt, \
                fews.timeserieskeys AS t \
                JOIN fews.timeseriesvaluesandflags AS s ON s.serieskey = t.serieskey \
                JOIN fews.stations ON t.locationkey = stations.stationkey \
                JOIN spatialfilter ON ST_Intersects(geom, spatialfilter.aoi), \
                params AS p \
                WHERE s.datetime <@ tsrange(p.period_start, p.period_end) AND \
                      t.parameterkey = pt.parameterkey \
                GROUP BY 1, 2, 3, 4 \
                ORDER BY 1, 2, 3 \
              ) \
            SELECT l.id AS station_id, \
              a.obs_year, \
              a.obs_month, \
              a.obs_value \
            FROM aggregates AS a JOIN fews.locations AS l ON a.locationkey = l.locationkey;" % (filterValue, filterValue, parameterid, username)

        dataQuery = "SELECT n.* \
            FROM crosstab ( \
             'select station_id,obs_month,trim(to_char((%s(obs_value)),''999.9999''))::REAL FROM monitoring.watertab group by 1,2 order by 1,2', \
             'select trim(to_char(generate_series(1,12),''09''))' \
             ) \
             AS n ( \
               locationid VARCHAR, jan REAL, feb REAL, mar REAL, apr REAL, may REAL, \
               jun REAL, jul REAL, aug REAL, sep REAL, oct REAL, nov REAL, dec REAL \
             );" % (aggFunction)

    if filterBy == "location" :

        viewQuery = "CREATE OR REPLACE VIEW monitoring.locationparametertab AS \
            WITH params as ( \
              SELECT %s AS locationkey \
            ), \
            pt AS ( \
              SELECT parameterkey \
              FROM fews.parameterstable \
              WHERE id = '%s' \
            ), \
            spatialfilter AS ( \
              SELECT ST_Union(geom) AS aoi \
              FROM app.user JOIN app.user_spatialfilter USING(userid) JOIN app.spatialfilter USING (gid) \
              WHERE username='%s' \
            ) \
            SELECT  p.locationkey, \
                    to_char(s.datetime, 'YYYY') AS obs_year, \
                    to_char(s.datetime, 'MM') AS obs_month, \
                    trim(to_char((s.scalarvalue),'999.9999'))::NUMERIC AS obs_value \
            FROM pt, \
            params AS p \
            JOIN fews.timeserieskeys AS t ON t.locationkey = p.locationkey \
            JOIN fews.timeseriesvaluesandflags AS s ON t.serieskey = s.serieskey \
            JOIN fews.stations ON t.locationkey = stations.stationkey \
            JOIN spatialfilter ON ST_Intersects(geom, spatialfilter.aoi) \
            WHERE t.parameterkey = pt.parameterkey \
            ORDER BY 1,2,3" % (filterValue, parameterid, username)

        dataQuery = "SELECT n.* from crosstab ( \
            'select obs_year,obs_month,trim(to_char((%s(obs_value)),''999.9999''))::NUMERIC from monitoring.locationparametertab group by 1,2 order by 1,2', \
            'select trim(to_char(generate_series(1,12),''09''))' \
            ) \
            AS n ( \
               locationid VARCHAR, jan REAL, feb REAL, mar REAL, apr REAL, may REAL, \
               jun REAL, jul REAL, aug REAL, sep REAL, oct REAL, nov REAL, dec REAL \
            )" % (aggFunction)

    query = pg.cursor(cursor_factory=RealDictCursor)
    query.execute(viewQuery)
    query.execute(dataQuery)

    result = json.dumps(query.fetchall())
    result = result.replace("null","0.0000000001")

    print ('{"success":"true", "records":',result.replace("'",'"'),'}')
else:
    print ('{"success":"false", "message":"Not authorized."}')
