#!/usr/bin/python3

import os
import cgi
import psycopg2
from datetime import datetime
from datetime import timedelta
import sys
import json
from psycopg2.extras import RealDictCursor
import xml.etree.ElementTree as xml
import urllib
from importlib.machinery import SourceFileLoader
import glob

def cgierror(message):
    print("Content-type: text/html")
    print()
    print(message)
    
def nonamespace(tag):
    i = tag.find('}')
    if i >= 0:
        tag = tag[i+1:]
    return tag

def printGetCapabilities():
    print("Content-type: text/xml")
    print()
    print('<?xml version="1.0" encoding="utf-8"?> \
           <wps:Capabilities service="WPS" version="1.0.0" xml:lang="en-CA" xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:wps="http://www.opengis.net/wps/1.0.0" \
           xmlns:ows="http://www.opengis.net/ows/1.1" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.opengis.net/wps/1.0.0 \
           http://schemas.opengis.net/wps/1.0.0/wpsGetCapabilities_response.xsd" updateSequence="1">')
    print('<wps:ProcessOfferings>')
    curpath = os.path.dirname(os.path.realpath( __file__ ))
    curfile = os.path.realpath( __file__ )
    curfile = curfile[1+len(curpath):-3]
    for identifier in glob.glob(curpath + '/*wps*.py'):
        identifier = identifier[1+len(curpath):-3]
        if identifier == curfile:
            continue # skip ourselves
        processor = SourceFileLoader(identifier, curpath + '/' + identifier + '.py').load_module()
        funcs = dir(processor)
        if not 'title' in funcs or not 'abstract' in funcs:
            continue
        implemented = 'true' if 'process' in funcs else 'false'
        print('<wps:Process wps:processVersion="1.1">')
        print('<ows:Identifier implemented="' + implemented + '">' + identifier + '</ows:Identifier>')
        print('<ows:Title>' + processor.title() + '</ows:Title>')
        print('<ows:Abstract>' + processor.abstract() + '</ows:Abstract>')
        print('</wps:Process>')
    print('</wps:ProcessOfferings>')
    print('</wps:Capabilities>')
    sys.stdout.flush()

def printDescribeProcess(identifier):
    if identifier is None:
        cgierror("DescribeProcess: bad identifier: None")
    elif not os.path.exists(identifier + ".py"):
        cgierror("DescribeProcess: identifier does not exist: '" + identifier + "'")
    else:
        processor = SourceFileLoader(identifier, identifier + ".py").load_module()
        print("Content-type: text/xml")
        print()
        print('<?xml version="1.0" encoding="utf-8"?> \
               <wps:ProcessDescriptions xmlns:wps="http://www.opengis.net/wps/1.0.0" xmlns:ows="http://www.opengis.net/ows/1.1" xmlns:xlink="http://www.w3.org/1999/xlink" \
               xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.opengis.net/wps/1.0.0 \
               http://schemas.opengis.net/wps/1.0.0/wpsDescribeProcess_response.xsd" service="WPS" version="1.0.0" xml:lang="en-CA"> \
               <ProcessDescription wps:processVersion="1.1" storeSupported="true" statusSupported="true">')
        print('<ows:Identifier>'+identifier+'</ows:Identifier>')
        funcs = dir(processor)
        if 'title' in funcs:
            print('<ows:Title>'+processor.title()+'</ows:Title>')
        else:
            print('<ows:Title></ows:Title>')
        if 'abstract' in funcs:
            print('<ows:Abstract>'+processor.abstract()+'</ows:Abstract>')
        else:
            print('<ows:Abstract></ows:Abstract>')
        print('<DataInputs>')
        if 'inputs' in funcs:
            for inparameter in processor.inputs():
                print('<Input minOccurs="0" maxOccurs="1">')
                print('<ows:Identifier>' + inparameter[0] + '</ows:Identifier>')
                print('<ows:Title>' + inparameter[1] + '</ows:Title>')
                print('<LiteralData><ows:DataType>' + inparameter[2] + '</ows:DataType></LiteralData>')
                print('</Input>')
        print('</DataInputs>')
        print('</ProcessDescription> \
               </wps:ProcessDescriptions>')
        sys.stdout.flush()

def printExecute(identifier, parameters):
    if identifier is None:
        cgierror("Execute: bad identifier: None")
    elif not os.path.exists(identifier + ".py"):
        cgierror("Execute: identifier does not exist: '" + identifier + "'")
    else:
        processor = SourceFileLoader(identifier, identifier + ".py").load_module()
        locations = parameters.get('locations')
        parameterid = parameters.get('parameterid')
        username = parameters.get('username')
        funcs = dir(processor)
        if not 'process' in funcs:
            cgierror("Execute: function '" + identifier + "' not implemented.")
        elif locations is None or locations.get('value') is None:
            cgierror("Execute: bad locations: None")
        elif parameterid is None or parameterid.get('value') is None:
            cgierror("Execute: bad parameterid: None")
        else:
            locations = locations['value'].split(',')
            parameterid = parameterid['value']
            username = username['value']
            startdate = parameters.get('startdate')
            enddate = parameters.get('enddate')
            if (not startdate is None) and (not enddate is None):
                startdate = startdate['value']
                enddate = enddate['value']
                startdate = datetime.strptime(startdate, "%d-%m-%Y")
                enddate = datetime.strptime(enddate, "%d-%m-%Y")
                enddate += timedelta(days=1)
                startdate = datetime.strftime(startdate, "%Y-%m-%d %H:%M:%S")
                enddate = datetime.strftime(enddate, "%Y-%m-%d %H:%M:%S")
            
            print("Content-type: application/json")
            print()
            
            parametername = ""
            unit = ""

            file = open("../.credentials/.postgres")
            connection_string = file.readline()
            pg = psycopg2.connect(connection_string)
            query = pg.cursor(cursor_factory=RealDictCursor)
            
            for locationid in locations:
                sql = "WITH spatialfilter AS \
                       (SELECT ST_Union(geom) AS aoi FROM app.user JOIN app.user_spatialfilter USING(userid) JOIN app.spatialfilter USING (gid) \
                        WHERE username='" + username + "') \
                       SELECT timeserieskeys.serieskey AS serieskey, parameterstable.name AS parametername, parametergroups.displayunit AS unit \
                        FROM fews.timeserieskeys \
                        JOIN fews.locations ON timeserieskeys.locationkey = locations.locationkey \
                        JOIN fews.parameterstable ON timeserieskeys.parameterkey = parameterstable.parameterkey \
                        JOIN fews.parametergroups ON parameterstable.groupkey = parametergroups.groupkey \
                        JOIN fews.stations ON locations.id = stations.station_id \
                        JOIN spatialfilter ON ST_Intersects(geom, spatialfilter.aoi) \
                        WHERE locations.id = '" + locationid + "' and parameterstable.id = '" + parameterid + "'"
                query.execute(sql)
                if query.rowcount == 0:
                    continue
                result = query.fetchall()
                key = result[0]['serieskey']    
                if parametername == "" and unit == "":
                    parametername = result[0]['parametername']
                    unit = result[0]['unit']
                    print (parametername + "," + unit + ",[", end='')
                else:
                    print(",", end='')
                print("{\"timeseriesid\": " + str(key) + ",\"locationid\": \"" + locationid + "\"", end='')
                sql = "SELECT datetime AS date, scalarvalue AS value FROM fews.timeseriesvaluesandflags WHERE serieskey = " + str(key)
                if (not startdate is None) and (not enddate is None):
                    sql += " AND datetime >= '" + startdate + "' AND datetime < '" + enddate + "'";
                sql += " ORDER BY 1"
                query.execute(sql)
                records = query.fetchall()
                parameters['locationid'] = locationid
                records = processor.process(identifier, parameters, records)
                result = json.dumps(records)
                print(", \"obs\":" + result.replace("'",'"') + "}", end='')
            print("]")

# main()

# GET/POST
querystring = os.environ.get('QUERY_STRING')
querystring = querystring.split('&')
query = {}
for qs in querystring:
    qs = qs.split('=')
    if len(qs) > 1:
        query[qs[0].lower()] = qs[1]
request = query.get('request')
identifier = ''
parameters = {}
if request is None: # POST
    contentlength = os.environ.get('CONTENT_LENGTH', 0)
    contentlength = int(contentlength)
    if contentlength > 0:
        root = sys.stdin.read(contentlength)
        root = xml.parse(root).getroot()
        request = nonamespace(root.tag)
        if request != 'GetCapabilities':
            identifier = root.find('{http://www.opengis.net/ows/1.1}Identifier').text
            if request == 'Execute':
                inputs = root.findall('{http://www.opengis.net/wps/1.0.0}DataInputs/{http://www.opengis.net/wps/1.0.0}Input')
                for input in inputs:
                    id = input.find('{http://www.opengis.net/ows/1.1}Identifier').text
                    parameter = {}
                    parameter['title'] = input.find('{http://www.opengis.net/ows/1.1}Title').text
                    parameter['value'] = input.find('{http://www.opengis.net/wps/1.0.0}Data/{http://www.opengis.net/wps/1.0.0}LiteralData').text
                    parameters[id] = parameter
else: # GET
    if request != 'GetCapabilities':
        identifier = query.get('identifier')
        if request == 'Execute':
            inputs = query.get('datainputs')
            if not inputs is None:
                inputs = urllib.parse.unquote(inputs)
                inputs = inputs.split('&')
                for input in inputs:
                    input = input.split('=')
                    parameter = {}
                    id = input[0]
                    parameter['value'] = input[1]
                    parameters[id] = parameter
            else:
                 cgierror("No inputs")

if not request is None:
    if request == 'GetCapabilities':
        printGetCapabilities()
    elif request == 'DescribeProcess':
        printDescribeProcess(identifier)
    elif request == 'Execute':
        ticket = query.get('ticket')
        ticket = urllib.parse.unquote(ticket)
        curpath = os.path.dirname(os.path.realpath( __file__ ))
        cas = SourceFileLoader("cas", curpath + "/../management/cas.py").load_module()
        if not cas.authorized(ticket,{"analysis" : "r"}):
            cgierror("Not authorized.")
        else:
            username = cas.getusername(ticket)
            parameters['username'] = {'value': username}
            printExecute(identifier, parameters)
    else:
        cgierror("Bad request: '" + request + "'")
else:
    printGetCapabilities()

